(*  BSL for MSP430FR5xxx with use the built-in BSL
    The ROM BSL is booted and a mass erase is done
    Then an empty password is sent
    Now the target is programmed, this takes about XX seconds.
    Finally the target is reset. The data is stored in XRAM!

  Connections from programmer to target MSP430FR5994
    Programmer         Target 
    --------------------------------
    P3.0 = Output   -> P2.1 RX
    P3.1 = Input    -> P2.0 TX
    P6.0 = Test     -> TEST/SBWTCK
    P6.1 = Reset    -> RST/NMI/SBWTDIO
    VCC             -> VCC
    GND             -> GND
    --------------------------------

  BSL commands  

    10 - Read Data Block    - Write data to MSP430
    11 - Password           - 20 Bytes password
    15 - Mass Erase         - Erase whole MSP430
    18 - TX Data Block      - Read data block back

Command format all bytes:

    | HDR | LNl | LNh | CMD | AL | AM | AH | DataBytes | CKL | CKH |

Response:
    | ACK | HDR | LNl | LNh | ... | CMD | MSG | CKL | CKH |

Messages:
    00 = ACK
    51 = Header incorrect
    52 = Checksum incorrect
    etc...

- Minimum delay after characters have been received is 1.2 millisec.
- A wrong password executes a mass erase
- So passwords don't need to be kept for these devices
- Just send two times the password

\ This one generates the correct CRC for the FRAM BSL
: CRC16-    ( a u -- crc16 )    \ With bits used in reversed order
    init-crc
    bounds ?do
        i c@ >crc  
    loop
    ." Res: "  crc> u. ;

*)

: C+!       ( n a -- )      >r r@ c@ +  r> c! ;
: INIT-CRC  ( -- )          -1 998 ! ;  \ CRCINIRES
: >CRC      ( b -- )        996 ! ;     \ Use CRCDIRB = reversed bits
: CRC>      ( -- crc )      998 @ ;     \ CRCINIRES

inside also
: FRACK?    ( -- fl )       bsl-key  dup .  0= ;    \ FRAM ack?
: FRAM-EMIT ( b -- )        dup >crc  bsl-emit ;    \ Emit to BSL with checksum
: FRAM-DATA ( lb hb -- )    fram-emit  fram-emit ;  \ Emit two bytes
: COMMAND}  ( -- )          crc>  split >r  bsl-emit  r> bsl-emit ; \ Send checksum

: {COMMAND      ( core-len command -- )         \ Start FRAM command frame
    80 bsl-emit  over bsl-emit swap >< bsl-emit \ -- adr +n command
    init-crc  fram-emit ;                       \ --

create RESPONSE 10 allot    \ Buffer to save BSL response
: !RESPONSE     ( b -- )    response >r  r@ count + c!  1 r> c+! ;
: >RESPONSE     ( -- )      bsl-key  dup >crc  !response ;
: .RESPONSE     ( -- )      response count  bounds ?do  i c@ u.  loop ;

: CORE-RESPONSE ( -- )      \ Receive BSL response
    0 response c!  >response  
    bsl-key dup !response  >response
    init-crc  2 + 0 ?do  >response  loop 
    crc> ?abort ;

: FRAM-ERASE    ( -- )      \ Erase whole FRAM memory
    cr ." Do mass erase "
    01 15 {command  command}  
    frack? 0= ?abort  core-response ;

: FRAM-PASSWORD ( -- )      \ Send password may be used for erase too
    cr ." Send password "  empty-pass
    pass  0020   dup 1+  11 {command
    bounds do  i c@ fram-emit  loop  command}
    1 ms  frack? 0= ?abort  core-response ;

\ Execute BSL command with data block
: {COMMAND?     ( xa +n core-len command -- flag )
    {command            \ Send command frame
    0 ?do               \ Send data block
        2dup i m+ x@    \ Get next data word        
        split fram-data \ Send as two bytes
    2 +loop
    command}            \ Finally checksum
    frack? 0= ;         \ Show if (no)Ack received? 

: XWRITE-FRAM   ( xa +n -- )    \ Write data from xa with length +n to BSL
    dup 4 +  10 {command?  ?abort  core-response ;

\ Write a binary part from XRAM using the FRAM BSL
: XWRITE-FR-PART ( xa1 adr +n -- xa2 )
    dup 80 mod >r               \ xa adr +n   R: mod    Length not a modula of 80
    r@ -  2dup + >r             \ xa adr +nm  R: adr1   Correct length, save adr1
    bounds ?do                  \ xa          Write first part
        i 80 xwrite-fram 80 m+  \ xa+80       In blocks of 80 bytes
    80 +loop
    r> r@ ?dup if               \ -- xa1 adr1 mod   Less the 80 left?
        xwrite-fram             \ -- xa1            Yes, write remaining part
    else  drop  then  r> m+ ;   \ -- xa2

\ Write binary from XRAM stored at '+n' using the FRAM BSL
: XWRITE-FRAM   ( +n -- )
    cr ." Write binary from XRAM "
    'entries 2x@ 2dup d.        \ -- xa             File addr.
    begin
        x@+ ?dup while          \ -- xa1 adr        Dest. addr.
        >r  x@+  r> swap        \ -- xa2 adr len    Length
        xwrite-fr-part          \ -- xa3
    repeat
    2drop ; 

forth definitions               \ Program the MSP430FR5xxx series 
: PROGRAM-FR5   ( +n -- )       \ Program the binary from dir entry '+n'
    setup-bsl  reset-to-bsl1    \ Start built-in BSL
    fram-erase  fram-password   \ Enter BSL and erase used flash
    xwrite-fram                 \ Write binary from XRAM
    10 ms  reset-target ;       \ Start new (binary) noForth

fresh
shield FR5\  freeze

\ End
